
//==========================================================
//                                                          
//     FILE:      dverk.cpp                                 
//                                                          
//==========================================================

#include <stdlib.h>
#include <math.h>
#include "dverk.h"
#define max(a, b)  (((a) > (b)) ? (a) : (b))
#define min(a, b)  (((a) < (b)) ? (a) : (b))

//----------------------------------------------------------
//          dverk.cpp           07.11.94  01.06.82          
//----------------------------------------------------------
//  IMSL ROUTINE NAME   - DVERK
//----------------------------------------------------------
//
//  COMPUTER            - IBM/DOUBLE
//
//  LATEST REVISION     - JUNE 1, 1982
//
//  PURPOSE             - DIFFERENTIAL EQUATION SOLVER - RUNGE
//                          KUTTA-VERNER FIFTH AND SIXTH ORDER METHOD
//
//  USAGE               -  DVERK
//
//  ARGUMENTS    N      - NUMBER OF EQUATIONS. (INPUT)
//               FCN    - NAME OF SUBROUTINE FOR EVALUATING FUNCTIONS.
//                          (INPUT)
//                          THE SUBROUTINE ITSELF MUST ALSO BE PROVIDED
//                            BY THE USER AND IT SHOULD BE OF THE
//                            FOLLOWING FORM
//    int FCN (
//               int       N       //
//             , double    X       //
//             , double *  Y       //
//             , double *  YPRIME  //
//            )
//                              REAL Y(N),YPRIME(N)
//                                   .
//                                   .
//                                   .
//                          FCN SHOULD EVALUATE YPRIME(1),...,YPRIME(N)
//                            GIVEN N,X, AND Y(1),...,Y(N).  YPRIME(I)
//                            IS THE FIRST DERIVATIVE OF Y(I) WITH
//                            RESPECT TO X.
//                          FCN MUST APPEAR IN AN EXTERNAL STATEMENT IN
//                            THE CALLING PROGRAM AND N,X,Y(1),...,Y(N)
//                            MUST NOT BE ALTERED BY FCN.
//               X      - INDEPENDENT VARIABLE. (INPUT AND OUTPUT)
//                          ON INPUT, X SUPPLIES THE INITIAL VALUE.
//                          ON OUTPUT, X IS REPLACED WITH XEND UNLESS
//                            ERROR CONDITIONS ARISE.  SEE THE DES-
//                            CRIPTION OF PARAMETER *IND.
//               Y      - DEPENDENT VARIABLES, VECTOR OF LENGTH N.
//                          (INPUT AND OUTPUT)
//                          ON INPUT, Y(1),...,Y(N) SUPPLY INITIAL
//                            VALUES.
//                          ON OUTPUT, Y(1),...,Y(N) ARE REPLACED WITH
//                            AN APPROXIMATE SOLUTION AT XEND UNLESS
//                            ERROR CONDITIONS ARISE.  SEE THE DES-
//                            CRIPTION OF PARAMETER *IND.
//               XEND   - VALUE OF X AT WHICH SOLUTION IS DESIRED.
//                          (INPUT)
//                          XEND MAY BE LESS THAN THE INITIAL VALUE OF
//                            X.
//               TOL    - TOLERANCE FOR ERROR CONTROL. (INPUT)
//                          THE SUBROUTINE ATTEMPTS TO CONTROL A NORM
//                            OF THE LOCAL ERROR IN SUCH A WAY THAT THE
//                            GLOBAL ERROR IS PROPORTIONAL TO TOL.
//                            MAKING TOL SMALLER IMPROVES ACCURACY AND
//                            MORE THAN ONE RUN, WITH DIFFERENT VALUES
//                            OF TOL, CAN BE USED IN AN ATTEMPT TO
//                            ESTIMATE THE GLOBAL ERROR.
//                          IN THE DEFAULT CASE (*IND=1), THE GLOBAL
//                            ERROR IS
//                                MAX(ABS(E(1)),...,ABS(E(N)))
//                            WHERE E(K)=(Y[K]-YT(K))/MAX(1,ABS(Y[K]))
//                            YT(K) IS THE TRUE SOLUTION, AND
//                            Y[K] IS THE COMPUTED SOLUTION AT XEND,
//                            FOR K=1,2,...,N.
//                            OTHER ERROR CONTROL OPTIONS ARE AVAILABLE.
//                            SEE THE DESCRIPTION OF PARAMETERS *IND AND
//                            C BELOW.
// *IND - INDICATOR. (INPUT AND OUTPUT) ON INITIAL ENTRY
//       *IND MUST BE SET EQUAL TO EITHER 1 OR 2.
//    *IND = 1 CAUSES ALL DEFAULT OPTIONS TO BE USED
//        AND ELIMINATES THE NEED TO SET SPECIFIC VALUES IN THE COMMUNICATIONS VECTOR C.
//    *IND = 2 ALLOWS OPTIONS TO BE SELECTED. IN THIS CASE, THE FIRST 9 COMPONENTS OF C
//        MUST BE INITIALIZED TO SELECT OPTIONS AS DESCRIBED BELOW.
//                        THE SUBROUTINE WILL NORMALLY RETURN WITH
//                          *IND = 3, HAVING REPLACED THE INITIAL VALUES
//                          OF X AND Y WITH, RESPECTIVELY, THE VALUE
//                          XEND AND AN APPROXIMATION TO Y AT XEND.
//                        THE SUBROUTINE CAN BE CALLED REPEATEDLY WITH
//                          NEW VALUES OF XEND WITHOUT CHANGING ANY
//                          OF THE OTHER PARAMETERS.
//                        THREE ERROR RETURNS ARE ALSO POSSIBLE, IN
//                          WHICH CASE X AND Y WILL BE THE MOST
//                          RECENTLY ACCEPTED VALUES.
//                          *IND = -3 INDICATES THAT THE SUBROUTINE WAS
//                            UNABLE TO SATISFY THE ERROR REQUIREMENT.
//                            THIS MAY MEAN THAT TOL IS TOO SMALL.
//                          *IND = -2 INDICATES THAT THE VALUE OF HMIN
//                            (MINIMUM STEP-SIZE) IS GREATER THAN HMAX
//                            (MAXIMUM STEP-SIZE), WHICH PROBABLY MEANS
//                            THAT THE REQUESTED TOL (WHICH IS USED IN
//                            THE CALCULATION OF HMIN) IS TOO SMALL.
//                          *IND = -1 INDICATES THAT THE ALLOWED MAXIMUM
//                            NUMBER OF FCN EVALUATIONS HAS BEEN
//                            EXCEEDED.  THIS CAN ONLY OCCUR IF OPTION
//                            CCC->lMaxFuncCalls, AS DESCRIBED BELOW, HAS BEEN USED.
// C - COMMUNICATIONS VECTOR OF LENGTH 24. (INPUT IF *IND != 1, AND OUTPUT).
//                          C IS USED TO SELECT OPTIONS AND TO RETAIN
//                            INFORMATION BETWEEN CALLS.  THE USER NEED
//                            NOT BE CONCERNED WITH THE FOLLOWING
//                            DESCRIPTION OF THE ELEMENTS OF C WHEN
//                            DEFAULT OPTIONS ARE USED (*IND=1).
//                            HOWEVER, WHEN IT IS DESIRED TO USE *IND=2
//                            AND SELECT OPTIONS, A BASIC UNDERSTANDING
//                            OF DVERK IS REQUIRED.  THE FOLLOWING
//                            PARAGRAPH DESCRIBES, BRIEFLY, THE BASIC
//                            TERMS.  FOR MORE DETAILS, SEE THE DOCUMENT
//                            REFERENCE.
//                            DVERK ADVANCES THE INDEPENDENT VARIABLE
//                            X ONE STEP AT A TIME UNTIL XEND IS
//                            REACHED.  THE SOLUTION IS COMPUTED AT
//  XTRIAL  HTRIAL            XTRIAL = X+HTRIAL ALONG WITH AN ERROR
//                            ESTIMATE EST.  IF EST IS LESS THAN OR
//                            EQUAL TO TOL (SUCCESSFUL STEP), THE STEP
//                            IS ACCEPTED AND X IS ADVANCED TO XTRIAL.
//                            IF EST IS GREATER THAN TOL (FAILURE)
//                            HTRIAL IS ADJUSTED AND THE SOLUTION IS
//  HMAG                      RECOMPUTED.  HMAG = ABS(HTRIAL) IS NEVER
//  HMAX                      ALLOWED TO EXCEED HMAX NOR IS IT ALLOWED
//  HSTART                    TO BECOME SMALLER THAN HMIN.  THE FIRST
//                            TRIAL STEP IS HSTART.  DURING THE
//                            COMPUTATION, A COUNTER (CCC->lSuccFails) IS
//                            INCREMENTED EACH TIME A TRIAL STEP FAILS
//                            TO PROVIDE A SOLUTION SATISFYING THE ERROR
//                            TOLERANCE.  ANOTHER COUNTER (CCC->lSuccSteps ) IS
//                            USED TO RECORD THE NUMBER OF SUCCESSFUL
//                            STEPS.  AFTER A SUCCESSFUL STEP,
//                            CCC->lSuccFails IS SET TO ZERO.
//                        OPTIONS.  IF THE SUBROUTINE IS ENTERED WITH
//                          *IND=2, THE FIRST 9 COMPONENTS OF THE
//                          COMMUNICATIONS VECTOR MUST BE INITIALIZED
//                          BY THE USER.  NORMALLY THIS IS DONE BY
//                          FIRST SETTING THEM ALL TO ZERO, AND THEN
//                          THOSE CORRESPONDING TO PARTICULAR OPTIONS
//                          ARE MADE NON-ZERO.
//               CCC->nErrThumb    - ERROR CONTROL INDICATOR.
//                          THE SUBROUTINE ATTEMPTS TO CONTROL A NORM
//                          OF THE LOCAL ERROR IN SUCH A WAY THAT THE
//                          GLOBAL ERROR IS PROPORTIONAL TO TOL.
//                          THE DEFINITION OF GLOBAL ERROR FOR THE
//                          DEFAULT CASE (*IND=1) IS GIVEN IN THE
//                          DESCRIPTION OF PARAMETER TOL.  THE DEFAULT
//                          WEIGHTS ARE 1/MAX(1,ABS(Y[K])).  WHEN *IND=2
//                          IS USED, THE WEIGHTS ARE DETERMINED
//                          ACCORDING TO THE VALUE OF CCC->nErrThumb .
//                          IF CCC->nErrThumb =1
//                                    THE WEIGHTS ARE 1
//                                    (ABSOLUTE ERROR CONTROL)
//                          IF CCC->nErrThumb =2
//                                    THE WEIGHTS ARE 1/ABS(Y[K])
//                                    FOR K=1,2,...,N.
//                                    (RELATIVE ERROR CONTROL)
//                          IF CCC->nErrThumb =3
//                                    THE WEIGHTS ARE
//                                    1/MAX(ABS(CCC->dErrFloor ),ABS(Y[K]))
//                                    FOR K=1,2,...,N.
//                                    (RELATIVE ERROR CONTROL, UNLESS
//                                    ABS(Y[K]) IS LESS THAN THE FLOOR
//                                    VALUE,ABS(CCC->dErrFloor))
//                          IF CCC->nErrThumb =4
//                                    THE WEIGHTS ARE
//                                    1/MAX(ABS(YFLOOR[K]),ABS(Y[K]))
//                                    FOR K=1,2,...,N.
//                                    (HERE INDIVIDUAL FLOOR VALUES
//                                    ARE USED)
//                                    IN THIS CASE,
//                                    YFLOOR[0],...,[N-1]
//                                    MUST BE INITIALIZED BY THE USER.
//                          IF CCC->nErrThumb =5
//                                    THE WEIGHTS ARE 1/ABS(YFLOOR[K])
//                                    FOR K=1,2,...,N.
//                                    IN THIS CASE,  THE DIMENSION OF C
//                                    MUST BE GREATER THAN OR EQUAL TO
//                                    N+30 AND YFLOOR[0],...,[N-1]
//                                    MUST BE INITIALIZED BY THE USER.
//                          FOR ALL OTHER VALUES OF CCC->nErrThumb,
//                                   INCLUDING
//                             CCC->nErrThumb =0
//                                    THE DEFAULT VALUES OF
//                                    THE WEIGHTS ARE TAKEN TO BE
//                                    1/MAX(1,ABS(Y[K]))
//                                    FOR K=1,2,...,N.
//               CCC->dErrFloor    - FLOOR VALUE.
//                          USED WHEN THE INDICATOR CCC->nErrThumb
//                          HAS THE VALUE 3.
//               CCC->dStepMin   - HMIN SPECIFICATION.
//                          IF NOT ZERO, THE SUB-
//                          ROUTINE CHOOSES HMIN TO BE ABS(CCC->dStepMin).
//                          OTHERWISE IT USES THE DEFAULT VALUE
//                          10*MAX(DWARF,RREB*MAX(NORM(Y)/TOL,ABS(X)))
//                          WHERE DWARF IS A VERY SMALL POSITIVE MACHINE
//                          NUMBER AND RREB IS THE RELATIVE ROUNDOFF
//                          ERROR BOUND.
//               CCC->dStepStart   - HSTART SPECIFICATION.
//                          IF NOT ZERO, THE SUB-
//                          ROUTINE WILL USE AN INITIAL HMAG EQUAL TO
//                          ABS(CCC->dStepStart),
//                          EXCEPT OF COURSE FOR THE RE-
//                          STRICTIONS IMPOSED BY HMIN AND HMAX.
//                          OTHERWISE IT USES THE DEFAULT VALUE
//                            HMAX*(TOL)**(1/6).
//               CCC->dScale   - SCALE SPECIFICATION.
//                          THIS IS INTENDED TO BE
//                          A MEASURE OF THE SCALE OF THE PROBLEM.
//                          LARGER VALUES OF SCALE TEND TO MAKE THE
//                          METHOD MORE RELIABLE, FIRST BY POSSIBLY RE-
//                          STRICTING HMAX (AS DESCRIBED BELOW) AND
//                          SECOND, BY TIGHTENING THE ACCEPTANCE
//                          REQUIREMENT.  IF CCC->dScale IS ZERO, A DEFAULT
//                          VALUE OF 1 IS USED. FOR LINEAR HOMOGENEOUS
//                          PROBLEMS WITH CONSTANT COEFFICIENTS, AN
//                          APPROPRIATE VALUE FOR SCALE IS A NORM OF
//                          THE ASSOCIATED MATRIX. FOR OTHER PROBLEMS,
//                          AN APPROXIMATION TO AN AVERAGE VALUE OF A
//                          NORM OF THE JACOBIAN ALONG THE TRAJEC-
//                          TORY MAY BE APPROPRIATE.
//               CCC->dStepMax   - HMAX SPECIFICATION.
//                          FOUR CASES ARE POSSIBLE,
//                          IF CCC->dStepMax != 0 AND CCC->dScale != 0,
//                        HMAX IS TAKEN
//                            TO BE MIN(ABS(CCC->dStepMax),2/ABS(CCC->dScale)).
//                          IF CCC->dStepMax != 0 AND CCC->dScale == 0,
//                        HMAX IS TAKEN
//                            TO BE ABS(CCC->dStepMax).
//                          IF CCC->dStepMax == 0 AND CCC->dScale != 0,
//                        HMAX IS TAKEN
//                            TO BE 2/ABS(CCC->dScale).
//                          IF CCC->dStepMax == 0 AND CCC->dScale == 0,
//                        HMAX IS GIVEN
//                            A DEFAULT VALUE OF 2.
// CCC->lMaxFuncCalls -  MAXIMUM NUMBER OF FUNCTION EVALUATIONS.
//    IF NOT ZERO, AN ERROR RETURN WITH *IND = -1 WILL BE CAUSED
//    WHEN THE NUMBER OF FUNCTION EVALUATIONS EXCEEDS ABS(CCC->lMaxFuncCalls).
//               CCC->nInterrupt1    - INTERRUPT NUMBER 1 .
//                          IF NOT ZERO, THE SUB-
//                          ROUTINE WILL INTERRUPT THE CALCULATIONS
//                          AFTER IT HAS CHOSEN ITS PRELIMINARY VALUE
//                          OF HMAG, AND JUST BEFORE CHOOSING HTRIAL
//                          AND XTRIAL IN PREPARATION FOR TAKING A STEP
//                          (HTRIAL MAY DIFFER FROM HMAG IN SIGN, AND
//                          MAY REQUIRE ADJUSTMENT IF XEND IS NEAR).
//                          THE SUBROUTINE RETURNS WITH *IND = 4, AND
//                          WILL RESUME CALCULATION AT THE POINT OF
//                          INTERRUPTION IF  RE-ENTERED WITH *IND = 4.
//               CCC->nInterrupt2    - INTERRUPT NUMBER 2.
//                          IF NOT ZERO, THE SUB-
//                          ROUTINE WILL INTERRUPT THE CALCULATIONS
//                          IMMEDIATELY AFTER IT HAS DECIDED WHETHER OR
//                          NOT TO ACCEPT THE RESULT OF THE MOST RECENT
//                          TRIAL STEP, WITH *IND = 5 IF IT PLANS TO
//                          ACCEPT, OR *IND = 6 IF IT PLANS TO REJECT.
//                          Y(*) IS THE PREVIOUSLY ACCEPTED RESULT,
//                          WHILE W(*,9) IS THE NEWLY COMPUTED TRIAL
//                          VALUE, AND W(*,2) IS THE UNWEIGHTED ERROR
//                          ESTIMATE VECTOR. THE SUBROUTINE WILL RESUME
//                          CALCULATIONS AT THE POINT OF INTERRUPTION
//                          ON RE-ENTRY WITH *IND = 5 OR 6.
//                          *IND MAY BE CHANGED BY THE USER IN ORDER TO
//                          FORCE ACCEPTANCE OF A STEP (BY CHANGING *IND
//                          FROM 6 TO 5) THAT WOULD OTHERWISE BE
//                          REJECTED, OR VICE VERSA.
//               NW     - ROW DIMENSION OF THE MATRIX W EXACTLY AS
//                          SPECIFIED IN THE DIMENSION STATEMENT
//                          IN THE CALLING PROGRAM. (INPUT)
//                          NW MUST BE GREATER THAN OR EQUAL TO N.
//               W      - WORKSPACE MATRIX.
//                          THE FIRST DIMENSION OF W MUST BE NW AND THE
//                          SECOND MUST BE GREATER THAN OR EQUAL TO 9.
//                          W MUST REMAIN UNCHANGED BETWEEN SUCCESSIVE
//                          CALLS DURING INTEGRATION.
//               IER    - ERROR PARAMETER. (OUTPUT)
//                        TERMINAL ERROR
//                          IER = 129, NW IS LESS THAN N OR TOL IS LESS
//                            THAN OR EQUAL TO ZERO.
//                          IER = 130, *IND IS NOT IN THE RANGE 1 TO 6.
//                          IER = 131, XEND HAS NOT BEEN CHANGED FROM
//                            PREVIOUS CALL OR X IS NOT SET TO
//                            THE PREVIOUS XEND VALUE.
//                          IER = 132, THE RELATIVE ERROR CONTROL
//                            OPTION (CCC->nErrThumb =2) WAS SELECTED AND
//                            ONE OF THE SOLUTION COMPONENTS
//                            IS ZERO.
//
//  PRECISION/HARDWARE  - SINGLE AND DOUBLE/H32
//                      - SINGLE/H36,H48,H60
//
//  REQD. IMSL ROUTINES - UERTST,UGETIO
//
//  NOTATION            - INFORMATION ON SPECIAL NOTATION AND
//                          CONVENTIONS IS AVAILABLE IN THE MANUAL
//                          INTRODUCTION OR THROUGH IMSL ROUTINE UHELP
//
//  REMARKS  1.  IN A TYPICAL SITUATION, DVERK IS CALLED
//               REPEATEDLY WITH A SEQUENCE OF VALUES FOR XEND.
//               AFTER EACH SUCH CALL, THE USER SHOULD INTERROGATE
//               *IND AND IER. ERROR CONDITIONS ARE SIGNALED WHEN
//               *IND IS LESS THAN ZERO AND/OR IER IS GREATER THAN
//               ZERO. CORRECTIVE ACTION (SUCH AS CHANGING CERTAIN
//               PARAMETER VALUES) MUST BE TAKEN PRIOR TO RE-ENTRY.
//           2.  WHEN ERROR CONDITIONS ARISE, IT IS OFTEN HELPFUL
//               TO EXAMINE COMPONENTS OF THE COMMUNICATIONS VECTOR
//               C. A SUMMARY FOLLOWS-
//
//               PRESCRIBED AT THE OPTION OF THE USER
//
//                 CCC->nErrThumb     ERROR CONTROL INDICATOR
//                 CCC->dErrFloor     FLOOR VALUE
//                 CCC->dStepMin      HMIN SPECIFICATION
//                 CCC->dStepStart    HSTART SPECIFICATION
//                 CCC->dScale        SCALE SPECIFICATION
//                 CCC->dStepMax      HMAX SPECIFICATION
//                 CCC->lMaxFuncCalls MAXIMUM NUMBER OF FCN EVALUATIONS
//                 CCC->nInterrupt1   INTERRUPT NUMBER 1
//                 CCC->nInterrupt2   INTERRUPT NUMBER 2
//
//               DETERMINED BY THE PROGRAM
//
//                 CCC->dRelRoundErrBound RREB
//                                    (RELATIVE ROUNDOFF ERROR BOUND)
//                 CCC->dDwarf            DWARF
//                                    (VERY SMALL MACHINE NUMBER)
//                 CCC->dWeightedNorm     WEIGHTED NORM OF Y
//                 CCC->dHMINInner        HMIN
//                 CCC->dStepMagn         HMAG
//                 CCC->dScaleInner       SCALE
//                 CCC->dHMAXInner        HMAX
//                 CCC->dXTrial           XTRIAL
//                 CCC->dStepTrial        HTRIAL
//                 CCC->dEstimation       EST
//                 CCC->dPrevXEND         PREVIOUS XEND
//                 CCC->nFlagXEND         FLAG FOR XEND
//                 CCC->lSuccSteps        NUMBER OF SUCCESSFUL STEPS
//                 CCC->lSuccFails        NUMBER OF SUCCESSIVE FAILURES
//                 CCC->lFuncCounter      NUMBER OF FCN EVALUATIONS
//
//               IF CCC->nErrThumb  = 4 OR 5,
//                 YFLOOR[0],...,[N-1] ARE FLOOR VALUES.
//
//           3.  PARAMETER NW GIVES THE ROW DIMENSION OF W EXACTLY AS
//               IT APPEARS IN THE DIMENSION STATEMENT IN THE CALLING
//               PROGRAM. IF ONLY ONE SYSTEM OF EQUATIONS IS BEING
//               SOLVED, NW NORMALLY WILL HAVE THE SAME VALUE AS N.
//               HOWEVER, IF MORE THAN ONE SYSTEM IS BEING HANDLED,
//               AND THEY ARE TO USE A COMMON WORKSPACE, W, ONE AFTER
//               THE OTHER, THE VALUE OF NW (AND HENCE, THE ROW
//               DIMENSION OF W IN THE CALLING PROGRAM) MUST BE AS
//               LARGE AS THE MAXIMUM VALUE OF THE INDIVIDUAL N VALUES.
//
//  COPYRIGHT           - 1978 BY IMSL, INC. ALL RIGHTS RESERVED.
//
//  WARRANTY            - IMSL WARRANTS ONLY THAT IMSL TESTING HAS BEEN
//                          APPLIED TO THIS CODE. NO OTHER WARRANTY,
//                          EXPRESSED OR IMPLIED, IS APPLICABLE.
//
//----------------------------------------------------------------------


DVERKCTRL::DVERKCTRL(){
  nErrThumb = 3; //! weights = 1/MAX(ABS(CCC->dErrFloor), ABS(Y(K)))
  dErrFloor      = 1.0e-6; //!  FLOOR VALUE.
  dStepMin       = 1.0e-7; //!  HMIN
  dStepStart     = 0.01;   //!  HSTART
  dScale         = 100.0;  //!  A MEASURE OF THE SCALE OF THE PROBLEM.
  dStepMax       = 0.1;   //!  HMAX SPECIFICATION.
  lMaxFuncCalls  = 1.0e6;  //!  MAXIMUM NUMBER OF FUNCTION EVALUATIONS.
  nInterrupt1    = 0;      //!  INTERRUPT NUMBER 1 .
  nInterrupt2    = 0;      //!  INTERRUPT NUMBER 2.
}


#define CONTINUE
  int dverk (
     int N          // int        N;
   , ODEFUNC FCN    // ODEFUNC    FCN;
   , double* X      // double   * X - (INPUT AND OUTPUT);
   , double* Y      // double   * Y;
   , double XEND    // double     XEND (INPUT);
   , double TOL     // double     TOL - ERROR CONTROL. (INPUT);
   , int* IND       // int      * IND - INDICATOR. (INPUT AND OUTPUT).
   , DVERKCTRL* CCC // DVERKCTRL* CCC (INPUT IF *IND != 1, AND OUTPUT).
   , double* YFLOOR // double   * YFLOOR (N).
   , double* W      // double   * W [ N * 9 ].
    )
{
  const long int  lZERO  = 0;
  const long int  lONE   = 1;
  const int       nZERO  = 0;
  const int       nONE   = 1;
  const int       nTWO   = 2;
  const int       nTHREE = 3;
  const int       nFOUR  = 4;
  const int       nFIVE  = 5;
  const int       nSEVEN = 7;
  const double    dZERO  = 0.0;
  const double    dONE   = 1.0;
  const double    dTWO   = 2.0;
  const double    dTEN   = 10.0;
  const double    HALF  = 0.5;
  const double    P9    = 0.9;
  const double    C4D15 = 0.2666666666666667;
  const double    C2D3  = 0.6666666666666667;
  const double    C5D6  = 0.8333333333333333;
  const double    C1D6  = 0.1666666666666667;
  const double    C1D15 = 0.6666666666666667e-1;
  const double    C2D96 = 120.4272910821709;
// DATA REPS/Z3410000000000000/  ! RREB (RELATIVE ROUNDOFF ERROR BOUND)
// DATA RTOL/Z0D10000000000000/  ! DWARF (VERY SMALL MACHINE NUMBER)
  const double     REPS = 1.0e-12;
  const double     RTOL = 1.0e-37;
      const double     RK01 = 0.1666666666666667e+00;
      const double     RK02 = 0.5333333333333333e-01;
      const double     RK03 = 0.2133333333333333e+00;
      const double     RK04 = 0.8333333333333333e+00;
      const double     RK05 = 0.2666666666666667e+01;
      const double     RK06 = 0.2500000000000000e+01;
      const double     RK07 = 0.2578125000000000e+01;
      const double     RK08 = 0.9166666666666667e+01;
      const double     RK09 = 0.6640625000000000e+01;
      const double     RK10 = 0.8854166666666667e+00;
      const double     RK11 = 0.2400000000000000e+01;
      const double     RK12 = 0.8000000000000000e+01;
      const double     RK13 = 0.6560457516339869e+01;
      const double     RK14 = 0.3055555555555556e+00;
      const double     RK15 = 0.3450980392156863e+00;
      const double     RK16 = 0.5508666666666667e+00;
      const double     RK17 = 0.1653333333333333e+01;
      const double     RK18 = 0.9455882352941176e+00;
      const double     RK19 = 0.3240000000000000e+00;
      const double     RK20 = 0.2337882352941176e+00;
      const double     RK21 = 0.2035465116279070e+01;
      const double     RK22 = 0.6976744186046512e+01;
      const double     RK23 = 0.5648179814561484e+01;
      const double     RK24 = 0.1373815676141258e+00;
      const double     RK25 = 0.2863022661036103e+00;
      const double     RK26 = 0.1441785567164738e+00;
      const double     RK27 = 0.7500000000000000e-01;
      const double     RK28 = 0.3899286987522282e+00;
      const double     RK29 = 0.3194444444444444e+00;
      const double     RK30 = 0.1350383631713555e+00;
      const double     RK31 = 0.1078329882677709e-01;
      const double     RK32 = 0.6980519480519481e-01;
      const double     RK33 = 0.6250000000000000e-02;
      const double     RK34 = 0.6963012477718360e-02;
      const double     RK35 = 0.6944444444444444e-02;
      const double     RK36 = 0.6138107416879795e-02;
      const double     RK37 = 0.6818181818181818e-01;
      const double     RK38 = 0.1078329882677709e-01;
      const double     RK39 = 0.6980519480519481e-01;
//                                 SPECIFICATIONS FOR LOCAL VARIABLES
      double* W1 = W;
      double* W2 = W1 + N;
      double* W3 = W2 + N;
      double* W4 = W3 + N;
      double* W5 = W4 + N;
      double* W6 = W5 + N;
      double* W7 = W6 + N;
      double* W8 = W7 + N;
      double* W9 = W8 + N;
      int K;
      double TEMP = 0.0;
      int IER = 0;
      int INDLoc = *IND;
      double   XLoc = *X;
      if ((*IND < 1)||(*IND > 6)) goto M290;
  switch (*IND){
    case 1 :goto   M5;
    case 2 :goto   M5;
    case 3 :goto  M40;
    case 4 :goto M145;
    case 5 :goto M265;
    case 6 :goto M265;
  }
//      goto (5,5,40,145,265,265), *IND
//  CASE 1 - INITIAL ENTRY (*IND  ==  1 OR 2) ABORT IF N > NW OR TOL <= 0
//
  M5: CONTINUE
    if (TOL <= dZERO) goto M295;
    if (*IND == 2) goto M15;
// INITIAL ENTRY WITHOUT OPTIONS (*IND == 1):
// SET CCC->nErrThumb TO CCC->nInterrupt2  EQUAL TO  0:
        CCC->nErrThumb      = nZERO;
        CCC->dErrFloor      = dZERO;
        CCC->dStepMin       = dZERO;
        CCC->dStepStart     = dZERO;
        CCC->dScale         = dZERO;
        CCC->dStepMax       = dZERO;
        CCC->lMaxFuncCalls  = lZERO;
        CCC->nInterrupt1    = nZERO;
        CCC->nInterrupt2    = nZERO;
    goto M30;
//                                 SUMMARY OF THE COMPONENTS OF THE
//                                   COMMUNICATIONS VECTOR
//                                   PRESCRIBED AT THE OPTION
//                                       OF THE USER
//
//                     CCC->nErrThumb     ERROR CONTROL INDICATOR
//                     CCC->dErrFloor     FLOOR VALUE
//                     CCC->dStepMin      HMIN SPECIFICATION
//                     CCC->dStepStart    HSTART SPECIFICATION
//                     CCC->dScale        SCALE SPECIFICATION
//                     CCC->dStepMax      HMAX SPECIFICATION
//                     CCC->lMaxFuncCalls MAX NO OF FCN EVALS
//                     CCC->nInterrupt1   INTERRUPT NO 1
//                     CCC->nInterrupt2   INTERRUPT NO 2
//
//                                   DETERMINED BY THE PROGRAM
//
//                CCC->dRelRoundErrBound RREB(REL ROUNDOFF ERROR BND)
//                CCC->dDwarf            DWARF (VERY SMALL MACH NO)
//                CCC->dWeightedNorm     WEIGHTED NORM Y
//                CCC->dHMINInner        HMIN
//                CCC->dStepMagn         HMAG
//                CCC->dScaleInner       SCALE
//                CCC->dHMAXInner        HMAX
//                CCC->dXTrial           XTRIAL
//                CCC->dStepTrial        HTRIAL
//                CCC->dEstimation       EST
//                CCC->dPrevXEND         PREVIOUS XEND
//                CCC->nFlagXEND         FLAG FOR XEND
//                CCC->lSuccSteps        NO OF SUCCESSFUL STEPS
//                CCC->lSuccFails        NO OF SUCCESSIVE FAILURES
//                CCC->lFuncCounter      NO OF FCN EVALS
//                   IF CCC->nErrThumb  = 4 OR 5,
//                YFLOOR (0),... ARE FLOOR VALUES.
//
   M15: CONTINUE
// INITIAL ENTRY WITH OPTIONS (*IND  ==  2):
// MAKE CCC->nErrThumb  TO CCC->nInterrupt2  NON-NEGATIVE:
        CCC->nErrThumb      =  abs (CCC->nErrThumb    );
        CCC->dErrFloor      = fabs (CCC->dErrFloor    );
        CCC->dStepMin       = fabs (CCC->dStepMin     );
        CCC->dStepStart     = fabs (CCC->dStepStart   );
        CCC->dScale         = fabs (CCC->dScale       );
        CCC->dStepMax       = fabs (CCC->dStepMax     );
        CCC->lMaxFuncCalls  = labs (CCC->lMaxFuncCalls);
        CCC->nInterrupt1    =  abs (CCC->nInterrupt1  );
        CCC->nInterrupt2    =  abs (CCC->nInterrupt2  );
  if ((CCC->nErrThumb == nFOUR)||(CCC->nErrThumb == nFIVE))
    {for (K = 0; K < N; K++) YFLOOR[K] = fabs (YFLOOR[K]);}
  M30: CONTINUE
      CCC->dRelRoundErrBound = REPS;
      CCC->dDwarf = RTOL;
// SET PREVIOUS XEND INITIALLY TO INITIAL VALUE OF X:
      CCC->dPrevXEND = XLoc;
      CCC->nFlagXEND    = nZERO;
      CCC->lSuccSteps   = lZERO;
      CCC->lSuccFails   = lZERO;
      CCC->lFuncCounter = lZERO;
      goto M45;
// CASE 2 - NORMAL RE-ENTRY (*IND  == 3)
// ABORT IF XEND REACHED, AND EITHER X CHANGED OR XEND NOT
   M40: CONTINUE
      if((CCC->nFlagXEND != nZERO) &&
          ((XLoc != CCC->dPrevXEND)||(XEND == CCC->dPrevXEND))) goto M285;
// RE-INITIALIZE FLAG:
      CCC->nFlagXEND = nZERO;
// CASE 3 - RE-ENTRY FOLLOWING AN INTERRUPT (*IND  ==  4 TO 6)
// TRANSFER CONTROL TO THE APPROPRIATE RE-ENTRY POINT.
// THIS HAS ALREADY BEEN HANDLED BY THE COMPUTED goto.
   M45: CONTINUE
// LOOP THROUGH THE FOLLOWING 4 STAGES,
// ONCE FOR EACH TRIAL STEP UNTIL THE OCCURRENCE OF ONE OF THE FOLLOWING
// (A) THE NORMAL RETURN (WITH *IND ==  3) ON REACHING XEND IN STAGE 4
// (B) AN ERROR RETURN (WITH *IND < 0) IN STAGE 1 OR STAGE 4
// (C) AN INTERRUPT RETURN (WITH *IND  == 4, 5 OR 6), IF REQUESTED,
// IN STAGE 1 OR STAGE 4
   M50 : CONTINUE
//  STAGE 1 - PREPARE - DO CALCULATIONS OF HMIN, HMAX, ETC.,
//  AND SOME PARAMETER CHECKING, AND END UP WITH SUITABLE VALUES OF HMAG,
//  XTRIAL AND HTRIAL IN PREPARATION FOR TAKING AN INTEGRATION STEP.
//  ERROR RETURN (WITH *IND=-1) IF NO OF FCN EVALS TOO GREAT
//
    if((CCC->lMaxFuncCalls != lZERO) && (CCC->lFuncCounter >= CCC->lMaxFuncCalls))
      { *IND = -1;  goto M9005; }
//   M55 : CONTINUE
//  CALCULATE SLOPE (ADDING 1 TO NO OF  FCN  EVALS) IF *IND != 6:
      if (*IND == 6) goto M60;
      FCN (N, XLoc, Y, W1);
      CCC->lFuncCounter++;
   M60: CONTINUE
//  CALCULATE HMIN - USE DEFAULT UNLESS VALUE PRESCRIBED
      CCC->dHMINInner = CCC->dStepMin;
      if (CCC->dStepMin != dZERO) goto M120;
// CALCULATE DEFAULT VALUE OF HMIN.
// FIRST CALCULATE WEIGHTED NORM Y - CCC->dWeightedNorm -
// AS SPECIFIED BY THE ERROR CONTROL INDICATOR CCC->nErrThumb:
   TEMP = dZERO;
  switch (CCC->nErrThumb)
  {
  case nONE:
// ABSOLUTE ERROR CONTROL - WEIGHTS ARE 1:
    for (K = 0; K < N; K++)
      TEMP = max(TEMP, fabs(Y[K]));
    CCC->dWeightedNorm = TEMP;
    break;
  case nTWO:
// RELATIVE ERROR CONTROL - WEIGHTS ARE
// 1/fabs (Y[K]) SO WEIGHTED NORM Y IS  1:
    CCC->dWeightedNorm = dONE;
    break;
  case nTHREE:
// WEIGHTS ARE 1/MAX(CCC->dErrFloor,ABS(Y[K])):
    for (K = 0; K < N; K++)
      TEMP = max (TEMP, fabs(Y[K]) / CCC->dErrFloor);
    CCC->dWeightedNorm = min(TEMP, dONE);
    break;
  case nFOUR:
// WEIGHTS ARE 1/MAX(YFLOOR[K],ABS(Y[K])):
    for (K = 0; K < N; K++)
      TEMP = max (TEMP, fabs(Y[K]) / YFLOOR[K]);
    CCC->dWeightedNorm = min (TEMP, dONE);
    break;
  case nFIVE:
// WEIGHTS ARE 1/YFLOOR[K]:
    for (K = 0; K < N; K++)
      TEMP = max (TEMP, (fabs (Y[K]) / YFLOOR[K]));
    CCC->dWeightedNorm = TEMP;
    break;
  default:
// DEFAULT CASE - WEIGHTS ARE: 1/MAX(1,ABS(Y[K])):
    for (K = 0; K < N; K++)
      TEMP = max (TEMP, fabs (Y[K]));
    CCC->dWeightedNorm = min (TEMP, dONE);
  }// switch;
  CCC->dHMINInner = dTEN * max(CCC->dDwarf, CCC->dRelRoundErrBound * max (CCC->dWeightedNorm/TOL
                           , fabs(XLoc)));
  M120: CONTINUE
// CALCULATE SCALE - USE DEFAULT UNLESS VALUE PRESCRIBED:
  CCC->dScaleInner = CCC->dScale;
      if (CCC->dScale == dZERO) CCC->dScaleInner = dONE;
// CALCULATE HMAX - CONSIDER 4 CASES:
// CASE 1 - BOTH HMAX AND SCALE PRESCRIBED:
      if ((CCC->dStepMax != dZERO) && (CCC->dScale != dZERO))
        CCC->dHMAXInner = min (CCC->dStepMax, dTWO/CCC->dScale);
// CASE 2 - HMAX PRESCRIBED, BUT SCALE NOT:
      if ((CCC->dStepMax != dZERO) && (CCC->dScale == dZERO))
        CCC->dHMAXInner = CCC->dStepMax;
// CASE 3 - HMAX NOT PRESCRIBED, BUT SCALE IS:
      if ((CCC->dStepMax == dZERO) && (CCC->dScale != dZERO))
        CCC->dHMAXInner = dTWO/CCC->dScale;
// CASE 4 - NEITHER HMAX NOR SCALE IS PROVIDED:
      if ((CCC->dStepMax == dZERO) && (CCC->dScale == dZERO))
        CCC->dHMAXInner = dTWO;
// ERROR RETURN (WITH *IND=-2) IF HMIN  >  HMAX:
      if (CCC->dHMINInner > CCC->dHMAXInner)
      {
         *IND = -2;
         goto M9005;
      }
// CALCULATE PRELIMINARY HMAG -  CONSIDER 3 CASES:
  if (*IND <= 2)
  {
// CASE 1 - INITIAL ENTRY - USE PRESCRIBED VALUE OF HSTART, IF ANY, ELSE DEFAULT
     CCC->dStepMagn = CCC->dStepStart;
     if (CCC->dStepStart == dZERO)
           CCC->dStepMagn = CCC->dHMAXInner * pow(TOL,C1D6);
  }
  else
  {
     if (CCC->lSuccFails <= nONE)
     {
// CASE 2 - AFTER A SUCCESSFUL STEP, OR AT MOST ONE FAILURE,
// USE MIN(2, 0.9*(TOL/EST)**(1/6))*HMAG,
// BUT AVOID POSSIBLE OVERFLOW. THEN AVOID REDUCTION BY MORE THAN HALF:
        TEMP = dTWO * CCC->dStepMagn;
        if (TOL < C2D96 * CCC->dEstimation)
           TEMP = P9 * pow ((TOL/CCC->dEstimation), C1D6)
                * CCC->dStepMagn;
        CCC->dStepMagn = max (TEMP, HALF * CCC->dStepMagn);
     }
     else
     {
// CASE 3 - AFTER TWO OR MORE SUCCESSIVE FAILURES
        CCC->dStepMagn = HALF * CCC->dStepMagn;
     }
  }
// CHECK AGAINST HMAX
     CCC->dStepMagn = min (CCC->dStepMagn, CCC->dHMAXInner);
// CHECK AGAINST HMIN
     CCC->dStepMagn = max (CCC->dStepMagn, CCC->dHMINInner);
// INTERRUPT NO 1 (WITH *IND=4) IF REQUESTED
     if (CCC->nInterrupt1 != nZERO)
     {
        *IND = 4;
        goto M9005;
     }
//                                RESUME HERE ON RE-ENTRY WITH *IND  ==  4 :
  M145: CONTINUE
//                                 CALCULATE HMAG, XTRIAL - DEPENDING
//                                   ON PRELIMINARY HMAG, XEND
//
      if (CCC->dStepMagn >= fabs (XEND - XLoc)) goto M150;
// DO NOT STEP MORE THAN HALF WAY TO  XEND:
      CCC->dStepMagn = min(CCC->dStepMagn, HALF * fabs(XEND - XLoc));
      if (XEND > XLoc)
         CCC->dXTrial = XLoc + CCC->dStepMagn;
      else
         CCC->dXTrial = XLoc - CCC->dStepMagn;
      goto M155;
  M150: CONTINUE
// HIT XEND EXACTLY:
      CCC->dStepMagn = fabs (XEND - XLoc);
      CCC->dXTrial  = XEND;
  M155: CONTINUE
// CALCULATE HTRIAL:
      CCC->dStepTrial  = CCC->dXTrial  - XLoc;
// STAGE 2 - CALCULATE YTRIAL (ADDING 7 TO NO OF FCN EVALS).
// W(*,2), ... W(*,8) HOLD INTERMEDIATE RESULTS NEEDED IN STAGE 3.
// W(*,9) IS TEMPORARY STORAGE UNTIL FINALLY IT HOLDS YTRIAL:
      for (K = 0; K < N; K++)
         W9[K] = Y[K] + CCC->dStepTrial  * W1[K] * RK01;
      FCN (N, XLoc + CCC->dStepTrial * C1D6, W9, W2);
      for (K = 0; K < N; K++)
         W9[K] = Y[K] + CCC->dStepTrial *
                   (
                      W1[K] * RK02
                    + W2[K] * RK03
                   );
      FCN (N, XLoc + CCC->dStepTrial * C4D15, W9, W3);
      for (K = 0; K < N; K++)
         W9[K] = Y[K] + CCC->dStepTrial *
                   (
                      W1[K] * RK04
                    - W2[K] * RK05
                    + W3[K] * RK06
                   );
      FCN (N, XLoc + CCC->dStepTrial * C2D3, W9, W4);
      for (K = 0; K < N; K++)
         W9[K] = Y[K] + CCC->dStepTrial *
                   (- W1[K] * RK07
                     + W2[K] * RK08
                     - W3[K] * RK09
                     + W4[K] * RK10
                   );
      FCN (N, XLoc + CCC->dStepTrial * C5D6, W9, W5);
      for (K = 0; K < N; K++)
         W9[K] = Y[K] + CCC->dStepTrial *
                   (
                      W1[K] * RK11
                    - W2[K] * RK12
                    + W3[K] * RK13
                    - W4[K] * RK14
                    + W5[K] * RK15
                  );
      FCN (N, XLoc+CCC->dStepTrial , W9, W6);
      for (K = 0; K < N; K++)
         W9[K] = Y[K] + CCC->dStepTrial *
                   (- W1[K] * RK16
                     + W2[K] * RK17
                     - W3[K] * RK18
                     - W4[K] * RK19
                     + W5[K] * RK20
                   );
      FCN (N, XLoc+CCC->dStepTrial *C1D15, W9, W7);
      for (K = 0; K < N; K++)
         W9[K] = Y[K] + CCC->dStepTrial *
                   (
                      W1[K] * RK21
                    - W2[K] * RK22
                    + W3[K] * RK23
                    - W4[K] * RK24
                    + W5[K] * RK25
                    + W7[K] * RK26
                  );
      FCN (N, XLoc+CCC->dStepTrial , W9, W8);
// CALCULATE YTRIAL, THE EXTRAPOLATED APPROXIMATION AND STORE IN W(*,9):
      for (K = 0; K < N; K++)
         W9[K] = Y[K] + CCC->dStepTrial *
                   (
                      W1[K] * RK27
                    + W3[K] * RK28
                    + W4[K] * RK29
                    + W5[K] * RK30
                    + W7[K] * RK31
                    + W8[K] * RK32
                  );
// ADD 7 TO THE NO OF FCN EVALS:
      CCC->lFuncCounter = CCC->lFuncCounter + nSEVEN;
// STAGE 3 - CALCULATE THE ERROR ESTIMATE EST.
// FIRST CALCULATE THE UNWEIGHTED ABSOLUTE ERROR ESTIMATE
// VECTOR (PER UNIT STEP) FOR THE UNEXTRAPOLATED APPROXIMATION
// AND STORE IT IN W(*,2).
// THEN CALCULATE THE WEIGHTED MAX NORM OF W(*,2)
// AS SPECIFIED BY THE ERROR CONTROL INDICATOR CCC->nErrThumb.
// FINALLY, MODIFY THIS RESULT TO PRODUCE EST,
// THE ERROR ESTIMATE (PER UNIT STEP)
// FOR THE EXTRAPOLATED APPROXIMATION YTRIAL.
// CALCULATE THE UNWEIGHTED ABSOLUTE ERROR ESTIMATE VECTOR.
      for (K = 0; K < N; K++)
         W2[K] =   W1[K] * RK33
                 + W3[K] * RK34
                 - W4[K] * RK35
                 + W5[K] * RK36
                 + W6[K] * RK37
                 - W7[K] * RK38
                 - W8[K] * RK39;
// CALCULATE THE WEIGHTED MAX NORM OF W(*,2)
// AS SPECIFIED BY THE ERROR CONTROL INDICATOR CCC->nErrThumb
     TEMP = dZERO;
     switch (CCC->nErrThumb)
     {
     case nONE:
// ABSOLUTE ERROR CONTROL:
        for (K = 0; K < N; K++)
           TEMP = max (TEMP, fabs(W2[K]));
        break;
     case nTWO:
// RELATIVE ERROR CONTROL:
        for (K = 0; K < N; K++)
        {
           if (Y[K] == dZERO ) goto M280;
           TEMP = max (TEMP, fabs(W2[K]/Y[K]));
        }
        break;
     case nTHREE:
// WEIGHTS ARE 1/MAX(CCC->dErrFloor,ABS(Y[K])):
        for (K = 0; K < N; K++)
           TEMP = max(TEMP, fabs(W2[K])/max(CCC->dErrFloor, fabs(Y[K])));
        break;
     case nFOUR:
// WEIGHTS ARE 1/MAX(YFLOOR[K],ABS(Y[K])):
        for (K = 0; K < N; K++)
           TEMP = max(TEMP, fabs(W2[K])/max(YFLOOR[K], fabs(Y[K])));
        break;
     case nFIVE:
// WEIGHTS ARE 1/YFLOOR[K]:
        for (K = 0; K < N; K++)
          TEMP = max (TEMP, fabs(W2[K]/YFLOOR[K]));
        break;
     default:
// DEFAULT CASE - WEIGHTS ARE 1/MAX(1,ABS(Y[K])):
      for (K = 0; K < N; K++)
         TEMP = max (TEMP, fabs (W2[K])/max(dONE, fabs (Y[K])));
   }// switch;
// CALCULATE EST - (THE WEIGHTED MAX NORM OF W(*,2))*HMAG*SCALE -
// EST IS INTENDED TO BE A MEASURE OF THE ERROR PER UNIT STEP IN YTRIAL:
      CCC->dEstimation  = TEMP * CCC->dStepMagn * CCC->dScaleInner;
// STAGE 4 - MAKE DECISIONS.
// SET *IND=5 IF STEP ACCEPTABLE, ELSE SET *IND=6:
      *IND = 5;
      if (CCC->dEstimation > TOL) *IND = 6;
// INTERRUPT NO 2 IF REQUESTED
      if (CCC->nInterrupt2 != nZERO) goto M9005;
// RESUME HERE ON RE-ENTRY WITH *IND  ==  5 OR 6 :
  M265: CONTINUE
     if (*IND == 6) goto M275;
// STEP ACCEPTED (*IND  ==  5), SO UPDATE X, Y FROM XTRIAL, YTRIAL,
// ADD 1 TO THE NO OF SUCCESSFUL STEPS,
// AND SET THE NO OF SUCCESSIVE FAILURES TO ZERO:
      XLoc = CCC->dXTrial;
      *X = XLoc;
      for (K = 0; K < N; K++)
         Y[K] = W9[K];
      CCC->lSuccSteps = CCC->lSuccSteps + lONE;
      CCC->lSuccFails = lZERO;
// RETURN(WITH *IND=3, XEND SAVED, FLAG SET) IF X  ==  XEND:
      if (XLoc != XEND) goto M50;
      *IND = 3;
      CCC->dPrevXEND = XEND;
      CCC->nFlagXEND = nONE;
      goto M9005;
  M275: CONTINUE
// STEP NOT ACCEPTED (*IND == 6), SO
// ADD 1 TO THE NO OF SUCCESSIVE FAILURES
      CCC->lSuccFails = CCC->lSuccFails + lONE;
// ERROR RETURN (WITH *IND=-3) IF HMAG  <=  HMIN
      if (CCC->dStepMagn > CCC->dHMINInner) goto M50;
      *IND = -3;
      goto M9005;
// END STAGE 4
// END LOOP
  M280: CONTINUE
// RELATIVE ERROR OPTION SELECTED AND Y[K] IS ZERO:
  IER = 132;
  goto M9000;
  M285: CONTINUE
// X OR XEND WAS NOT CHANGED FROM PREVIOUS CALL
      IER = 131;
      goto M9000;
  M290: CONTINUE
//                                 *IND OUT OF RANGE
  IER = 130;
  goto M9000;
  M295: CONTINUE
//                                 TOL <= 0
  IER = 129;
//                                 BEGIN ABORT ACTION
  M9000: CONTINUE
//       UERTST (IER,6HDVERK)
  M9005: CONTINUE
  return(IER);
}

//==========================================================
//                                                          
//  ENDFILE:      dverk.cpp                                 
//                                                          
//==========================================================


